Skip to contents

Produces coefficient plots tailored to the model family:

  • sofr_bayes(), fcox_bayes(): one curve plot per functional predictor coefficient \(\beta(s)\), with pointwise and/or CMA credible bands.

  • fosr_bayes(): one curve plot per scalar predictor coefficient function \(\alpha_p(t)\).

  • fofr_bayes(): curve plots for scalar predictor coefficient functions \(\alpha_p(t)\) (if any), followed by heatmap plots for each bivariate coefficient surface \(\beta_q(s, t)\) (posterior mean) from the functional predictors.

  • fpca_bayes(): posterior mean function \(\mu(t)\) with a pointwise credible band; a combined plot of the (fixed) FPC eigenfunctions \(\phi_j(t)\); a point-and-error-bar plot of the posterior of the eigenvalue SDs \(\lambda_j\); and a histogram of the residual-SD posterior \(\sigma_\epsilon\).

Usage

# S3 method for class 'refundBayes'
plot(x = NULL, ..., prob = 0.95, include = "both")

Arguments

x

A fitted object returned by sofr_bayes(), fosr_bayes(), fofr_bayes(), fcox_bayes(), or fpca_bayes().

...

Other parameters

prob

Coverage probability for the credible interval(s). Defaults to 0.95.

include

Type of interval to include. "pointwise" produces pointwise credible intervals; "CMA" produces the CMA credible band; "both" produces both. Defaults to "both". Only used for sofr_bayes() / fcox_bayes() curve plots.

Value

A named list of ggplot objects. For FoFR, scalar-predictor curves are named scalar_<p> and bivariate-predictor heatmaps are named bivar_<q>. For FPCA, the plots are named mu, efunctions, evalues, and sigma.