Plot the estimated functional coefficients with the corresponding credible interval(s).
Source:R/refundBayes_plot.R
plot.refundBayes.RdPlot the estimated functional coefficients with the corresponding credible interval(s).
Usage
# S3 method for class 'refundBayes'
plot(x = NULL, ..., prob = 0.95, include = "both")Arguments
- x
A fitted object returned by
sofr_bayes().- ...
Other parameters
- prob
Coverage probability for the credible interval(s). Defaults to 0.95.
- include
Type of interval to include.
"pointwise"produces pointwise credible intervals;"CMA"produces the CMA credible band;"both"produces both. Defaults to"both".